---------------------------------------------------------------------
-- Transformer - Talent/Gear switcher by Acuzod - acuzod@gmail.com --
---------------------------------------------------------------------

--------------------
-- SavedVariables --
--------------------
 
Transformer_Settings = {
	MinimapPos = 45,
	Transformer_Active_Name = nil,
    Transformer_Active_IconTexture = nil,
    Transformer_Inactive_Name = nil
}

---------------------
-- Local Variables --
---------------------

-- Filters Based on DualSpeccer by Jeffrey0
local filter1 = string.gsub(ERR_SPELL_UNLEARNED_S:gsub('%.', '%.'), '%%s', '(.*)')
local filter2 = string.gsub(ERR_LEARN_ABILITY_S:gsub('%.', '%.'), '%%s', '(.*)')
local filter3 = string.gsub(ERR_LEARN_SPELL_S:gsub('%.', '%.'), '%%s', '(.*)')


-------------------
-- Addon Startup --
-------------------

function Transformer_OnLoad()
    this:RegisterEvent("PLAYER_ALIVE")
    this:RegisterEvent("ADDON_LOADED")
end


-------------------
-- Event Watcher --
-------------------

function Transformer_OnEvent() 
 
    
    if (event == "PLAYER_ALIVE") then
      	Transformer_LoadTalentsInfo()
      	this:UnregisterEvent("PLAYER_ALIVE")
    end
    
    if (event == "ADDON_LOADED" and arg1 == "Transformer") then
        Transformer_MinimapButton_Icon:SetTexture(Transformer_Settings.Transformer_Active_IconTexture)
        Transformer_MinimapButton_Reposition()
    end
          
    if (event == "UNIT_SPELLCAST_SUCCEEDED") and arg1 == "player" then
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_START")
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
        Transformer_LoadTalentsInfo()
        PlaySoundFile("Interface\\AddOns\\Transformer\\transformer.mp3")
        UseEquipmentSet(Transformer_Settings.Transformer_Active_Name)
    end
    
    if ((event == "UNIT_SPELLCAST_INTERRUPTED" or event == "UNIT_SPELLCAST_FAILED")  and arg1 == "player") then
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_START")
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
        print("Transformer: Transformation interrupted")
    end
             
end

------------------------------------------
-- Minimap Button Position and Movement --
------------------------------------------

function Transformer_MinimapButton_Reposition()
	Transformer_MinimapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(Transformer_Settings.MinimapPos)),(80*sin(Transformer_Settings.MinimapPos))-52)
end

-- Only while the button is dragged this is called every frame
function Transformer_MinimapButton_DraggingFrame_OnUpdate()

	local xpos,ypos = GetCursorPosition()
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom()

	xpos = xmin-xpos/UIParent:GetScale()+70 -- get coordinates as differences from the center of the minimap
	ypos = ypos/UIParent:GetScale()-ymin-70

	Transformer_Settings.MinimapPos = math.deg(math.atan2(ypos,xpos)) -- save the degrees we are relative to the minimap center
	Transformer_MinimapButton_Reposition() -- move the button
end

-------------------------------
-- Minimap Button Left Click --
-------------------------------'

-- Put your code that you want on a minimap button click here.  arg1="LeftButton", "RightButton", etc
function Transformer_MinimapButton_OnClick()
    if arg1 == "LeftButton" then
            
        if GetNumTalentGroups(0) == 2 then
        
            if GetActiveTalentGroup(0) == 1 then
                SetActiveTalentGroup(2)
            else
                SetActiveTalentGroup(1)
            end     
        end
    end 
end

-----------------------
-- Load Talents Info --
-----------------------

function Transformer_LoadTalentsInfo()
 
    local numPoints
    local Primary_Name
    local Primary_IconTexture
    local Secondary_Name
    local Secondary_IconTexture
    
    -- Get primary talent spec info
    numPoints = 0
    for i = 1, GetNumTalentTabs() do
        name, iconTexture, pointsSpent, background, previewPointsSpent = GetTalentTabInfo(i,0,0,1)
        if pointsSpent > numPoints then
        	Primary_Name = name
	        Primary_IconTexture = iconTexture
	        numPoints = pointsSpent
        end 
    end
        
    -- Get secondary talent spec info if exists  
    numPoints = 0
    if GetNumTalentGroups(0) == 2 then 
     
        for i = 1, GetNumTalentTabs() do
            name, iconTexture, pointsSpent, background, previewPointsSpent = GetTalentTabInfo(i,0,0,2)
            if pointsSpent > numPoints then
        	    Secondary_Name = name
	            Secondary_IconTexture = iconTexture
	            numPoints = pointsSpent
            end 
        end   
    end  
    
    -- Set current group info    
    if GetActiveTalentGroup(0) == 1 then
        Transformer_Settings.Transformer_Active_Name = Primary_Name
        Transformer_Settings.Transformer_Active_IconTexture = Primary_IconTexture
        Transformer_Settings.Transformer_Inactive_Name = Secondary_Name
    else
        Transformer_Settings.Transformer_Active_Name = Secondary_Name
        Transformer_Settings.Transformer_Active_IconTexture = Secondary_IconTexture
        Transformer_Settings.Transformer_Inactive_Name = Primary_Name
    end
        
    Transformer_MinimapButton_Icon:SetTexture(Transformer_Settings.Transformer_Active_IconTexture)
    print("Transformer: You are " .. Transformer_Settings.Transformer_Active_Name)
end

--------------------------------
-- Set Minimap Button Tooltip --
--------------------------------

function Transformer_MinimapButton_OnEnter(self)
	if (self.dragging) then
		return
	end
	
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	GameTooltip:SetText("Transformer")
	
    GameTooltip:AddLine("Current spec: " .. Transformer_Settings.Transformer_Active_Name, 1, 1, 1)

    if GetNumTalentGroups(0) == 1 then
	    GameTooltip:AddLine("No secondary spec available",1)
	else
	    GameTooltip:AddLine("Click to change to: " .. Transformer_Settings.Transformer_Inactive_Name, 1, 1, 1)
	end   
	
	GameTooltip:Show()
end

------------------------
-- Talent Change Hook --
------------------------

local orgSetActiveTalentGroup = _G.SetActiveTalentGroup

function SetActiveTalentGroup(...)
    print("Transformer: Going " .. Transformer_Settings.Transformer_Inactive_Name .. "...")
	transformerFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    transformerFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
    transformerFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
    transformerFrame:RegisterEvent("UNIT_SPELLCAST_START")	
    ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
	return orgSetActiveTalentGroup(...)
end


function filter(self, event, msg)
    if strfind(msg, filter1) or strfind(msg, filter2) or strfind(msg, filter3) then
	    return true
    end
end
	
	